from core.report.connection import ConnectionSearcher
from core.admin import admin_main
from core.config import getConfig
from core.user import user_main
from core.ibs_exceptions import *
from core.ras import ras_main
from ibs_lib.ibs_csv import IBSngDialect

import os, time, shutil, csv
from os.path import join, split, splitext
from zipfile import ZipFile, ZIP_DEFLATED
import ConfigParser
from ConfigParser import ParsingError

from pprint import pprint

conf_path = splitext(__file__)[0] + '.conf'

config = ConfigParser.RawConfigParser()
config.read(conf_path)

date_type = 'gregorian'
sleep_time = 0
default_root_dir = '/var/backup/archive'
split_conds = ('ras_desc', 'group_name', 'not_group_name')## FIXME these conditions must be converted from string (with commas) to list

defaults = dict(
    compress_file = False,
    cols = ['user_id', 'username', 'login_time_formatted', 'logout_time_formatted', 'remote_ip', 'caller_id', 'mac',
            'called_number', 'ras_id'],
    details_cols = ['dnis', 'called_ip', 'serial', 'ras_ip'],
    ordered_cols_int = ['login_time_formatted', 'logout_time_formatted', 'remote_ip', 'caller_id', 'mac', 'username'],
    ordered_cols_voip = ['login_time_formatted', 'logout_time_formatted', 'ras_ip', 'caller_id', 'called_ip',
                         'called_number', 'dnis', 'username', 'serial'],
    add_phone = False,
    default_called_ip = '',
    exclude_empty = [],## remote_ip for internet (non-IN)
    alias_common = {
        'login_time_formatted': 'Start Date',
        'logout_time_formatted': 'End Date',
        'user_id': 'User ID',
        'username': 'UserName',
        'serial': 'Serial',
        'remote_ip': 'IP Address',
        'mac': 'Mac Address',
        'dnis': 'DNIS',
        'ras_ip': 'RAS IP',
        'group_name': 'Group Name',
    },
    alias_int = {
        'caller_id': 'Tel Number',
    },
    alias_voip = {
        'username': 'Pin Code',    
        'dnis': 'Calling Number(DNIS)',
        'ras_ip': 'Source IP',
        'caller_id': 'Caller ID',## or Caller Number FIXME
        'called_ip': 'Destination IP',
        'called_number': 'Called Number',
    },
)

def boolValue(value):
    value = value.lower()
    if value in ('1', 't', 'true'):
        return True
    elif value in ('0', 'f', 'false'):
        return False
    else:
        raise ValueError('bad boolean value %s'%value)



for section in config.sections():
    if section=='GENERAL':
        for (key, value) in config.items(section):
            if key=='date_type':
                ch = value.lower()[0]
                if ch=='g':
                    date_type = 'gregorian'
                elif ch=='j':
                    date_type = 'jalali'
                else:
                    raise ParsingError('bad date_type=%r'%value)
            elif key=='sleep':
                sleep_time = float(value)
            elif key=='compress_file':
                defaults['compress_file'] = boolValue(value)
            elif key=='default_called_ip':
                defaults['default_called_ip'] = value
    else:
        data = dict(config.items(section))
        if 'dir' in data:
            direc = data['dir']
            del data['dir']
        else:
            direc = join(default_root_dir, section)
        if 'service' in data:
            service = data['service']
            if service.lower()=='internet':
                service = 'Internet'
            elif service.lower()=='voip':
                service = 'VoIP'
            else:
                raise ParsingError('bad serivce=%r'%service)
            del data['service']
        else:
            service = 'Internet'

        compress_file = defaults['compress_file']
        add_phone = defaults['add_phone']
        default_called_ip = defaults['default_called_ip']
        exclude_empty = defaults['exclude_empty']
        alias = defaults['alias_common']
        if service=='Internet':
            ordered_cols = defaults['ordered_cols_int']
            alias.update(defaults['alias_int'])
        else:
            ordered_cols = defaults['ordered_cols_voip']
            alias.update(defaults['alias_voip'])

        login_time_from = None
        login_time_to = None
        filename = None
        
        conds = {
            'login_time_from_unit': date_type,
            'login_time_to_unit': date_type,
            'service': service,
            'successful': 'yes',
        }
        for (key, value) in data.items():
            if key=='login_time_from':
                login_time_from = value
            elif key=='login_time_to':
                login_time_to = value
            elif key=='filename':
                filename = value
            elif key=='compress_file':
                compress_file = boolValue(value)
            elif key=='columns':
                ordered_cols = []
                for part in value.split(','):
                    part = part.strip()
                    if part in ('login_time', 'logout_time'):
                        ordered_cols.append(part+'_formatted')
                    else:
                        ordered_cols.append(part)
            elif key.startswith('cond_'):
                key = key[5:]
                if key in split_conds:
                    value = [x.strip() for x in value.split(',')]
                conds[key] = value
            elif key.startswith('alias_'):
                key = key[6:]
                alias[key] = value
            elif key=='exclude_empty':
                exclude_empty = [x.strip() for x in value.split(',')]
            elif key=='add_phone':
                try:
                    add_phone = boolValue(value)
                except:
                    raise ParsingError('bad boolean value add_phone=%s'%value)
            elif key=='default_called_ip':
                default_called_ip = value

        filename_range = login_time_from!=None or login_time_to!=None ## if one of start or end is given due to config file
        t = time.time()
        if not login_time_from:
            login_time_from = '%.4d-%.2d-%.2d'%tuple(time.localtime(t-24*3600))[:3]## yesterday
        if not login_time_to:
            login_time_to = '%.4d-%.2d-%.2d'%tuple(time.localtime(t))[:3]## today
        if not filename:
            if filename_range:
                filename = login_time_from.split(' ')[0] + '_' + login_time_to.split(' ')[0] + '.csv'
            else:
                filename = login_time_from + '.csv'
        filepath = join(direc, filename)

        conds['login_time_from'] = login_time_from
        conds['login_time_to'] = login_time_to

        try:
            os.makedirs(direc)
        except:
            pass

        cols = []
        details_cols = []
        for col in ordered_cols:
            if col in defaults['cols']:
                cols.append(col)
            elif col in defaults['details_cols']:
                details_cols.append(col)
        if 'ras_ip' in details_cols and not 'ras_id' in cols:
            cols.append('ras_id')
        all_cols = cols + details_cols

        ConnectionSearcher(
            conds,
            admin_main.getAdminLoader().getAdminByID(0), 
            'ADMIN',
        ).writeConnectionLogsToFile(
            filepath, 
            cols, 
            details_cols, 
            'connection_log_id', 
            False, 
            date_type,
        )
        
        rows = list(csv.reader(open(filepath, "r"), dialect=IBSngDialect))
        rows.pop(0)## get and remove header

        if service=='VoIP' and 'ras_ip' in all_cols:
            ras_ip_index = all_cols.index('ras_ip')
            ras_id_index = all_cols.index('ras_id')
            for row in rows:
                try:
                    row[ras_ip_index] = ras_main.getLoader().getRasByID(int(row[ras_id_index])).getRasIP()
                except Exception, e:
                    pass

        if add_phone and 'caller_id' in all_cols:## and service=='Internet' ## FIXME
            caller_id_index = all_cols.index('caller_id')
            
            for row in rows:
                try:
                    row[caller_id_index]
                except IndexError:
                    toLog('row=%s, caller_id_index=%s'%(row, caller_id_index), LOG_ERROR)
                    raise

                if not row[caller_id_index]:
                    try:
                        row[caller_id_index] = user_main.getUserPool().getUserByID(row[0]).getUserAttrs()['phone']
                    except:
                        pass

        if 'serial' in all_cols:
            serial_index = all_cols.index('serial')
            for row in rows:
                try:
                    serial = user_main.getUserPool().getUserByID(row[0]).getUserAttrs()['serial']
                except Exception, e:
                    if not 'ATTR_NOT_FOUND' in str(e):
                        open('/tmp/serial/%s'%row[0], 'w').write(str(e))
                    serial = ''
                row[serial_index] = serial

        if service=='VoIP' and 'called_ip' in all_cols and default_called_ip:
            called_ip_index = all_cols.index('called_ip')
            for row in rows:
                if not row[called_ip_index]:
                    row[called_ip_index] = default_called_ip

        for col in exclude_empty:
            try:
                index = all_cols.index(col)
            except:
                pass
            else:
                row_num = len(rows)
                row_i = 0
                while row_i < row_num:
                    if rows[row_i][index]:
                        row_i += 1
                    else:
                        rows.pop(row_i)
                        row_num -= 1

        writer = csv.writer(open(filepath, 'w+'), dialect='excel')

        writer.writerow([alias[col] for col in ordered_cols])
        ordered_indexes = [all_cols.index(col) for col in ordered_cols]
        for row in rows:
            writer.writerow([row[i] for i in ordered_indexes])
        del writer

        if compress_file:
            zip_file = ZipFile(splitext(filepath)[0] + '.zip', 'w', ZIP_DEFLATED)
            zip_file.write(filepath, filename)
            zip_file.close()
            os.remove(filepath)
        
        time.sleep(sleep_time)






